Namespace My

    ' The following events are availble for MyApplication:
    ' 
    ' Startup: Raised when the application starts, before the startup form is created.
    ' Shutdown: Raised after all application forms are closed.  This event is not raised if the application terminates abnormally.
    ' UnhandledException: Raised if the application encounters an unhandled exception.
    ' StartupNextInstance: Raised when launching a single-instance application and the application is already active. 
    ' NetworkAvailabilityChanged: Raised when the network connection is connected or disconnected.
    Partial Friend Class MyApplication
        ' Catch any unhandled exceptions.
        Private Sub MyApplication_UnhandledException(ByVal sender As Object, ByVal e As Microsoft.VisualBasic.ApplicationServices.UnhandledExceptionEventArgs) Handles Me.UnhandledException
            ProcessUnhandledException(sender, e)
        End Sub

        ' Deal with an unhandled exception.
        Public Sub ProcessUnhandledException(ByVal sender As Object, ByVal e As Microsoft.VisualBasic.ApplicationServices.UnhandledExceptionEventArgs)
            Dim msg As String = ""

            ' Try to make a message with a stack trace.
            Try
                Dim stack_trace As New StackTrace(1, True)
                msg = vbCrLf & Now & vbCrLf & stack_trace.ToString()
            Catch ex As Exception
            End Try

            ' Try making a simpler message.
            If msg.Length = 0 Then
                Try
                    msg = vbCrLf & Now & "Unable to save stack trace."
                Catch ex As Exception
                End Try
            End If

            ' Try to email the message.
            Dim email_worked As Boolean = False
            Try
                SendEmail("mailserver", _
                    "myapplication@somewhere.com", _
                    "developers@somewhere.com", _
                    "Automated Error Report", _
                    msg)
                email_worked = True
            Catch ex As Exception
            End Try

            ' Try writing to a log file.
            ' Note: You may want to do this even if the email works.
            Dim log_worked As Boolean = False
            If Not email_worked Then
                Try
                    My.Application.Log.WriteEntry(msg)
                    log_worked = True
                Catch ex As Exception
                End Try
            End If

            ' See if we succeeded.
            If email_worked Then
                ' Tell the user that we emailed a message to the developers.
                e.ExitApplication = _
                    MessageBox.Show("Unexpected error: " & vbCrLf & _
                        e.Exception.Message & vbCrLf & vbCrLf & _
                        "Automatically emailed a report to the developers." & vbCrLf & vbCrLf & _
                        "Do you want to end the application?", _
                        "Unexpected Error", _
                        MessageBoxButtons.YesNo, _
                        MessageBoxIcon.Question) = DialogResult.Yes
            ElseIf log_worked Then
                ' Tell the user that we saved a log entry.
                e.ExitApplication = _
                    MessageBox.Show("Unexpected error: " & vbCrLf & _
                        e.Exception.Message & vbCrLf & vbCrLf & _
                        "Automatically saved a log entry. Please notify the developers." & vbCrLf & vbCrLf & _
                        "Do you want to end the application?", _
                        "Unexpected Error", _
                        MessageBoxButtons.YesNo, _
                        MessageBoxIcon.Question) = DialogResult.Yes
            Else
                ' We did not save any information.
                ' Ask the user to call support.
                MessageBox.Show("Unexpected error: " & vbCrLf & _
                    e.Exception.Message & vbCrLf & vbCrLf & _
                    "Unable to save error information. Please call the developers immediately." & vbCrLf & vbCrLf & _
                    "After talking to the developers, click OK to close the application.", _
                    "Unexpected Error", _
                    MessageBoxButtons.OK, _
                    MessageBoxIcon.Exclamation)
                e.ExitApplication = True
            End If
        End Sub
    End Class
End Namespace

